<?php

/**
 * @filesource: rgraph_options.php
 * @author:     Joachim Schmidt - joachim.schmidt@jschmidt-systemberatung.de
 * @copyright:  Copyright (C) 2016 Joachim Schmidt. All rights reserved.
 * @license:    GNU/GPL
 * @version:    1.0
 * @abstract:   json php-helper class for rgraph charts package
 *
 */
include_once 'rgraph_valid_options.php';

class rgraph_options
{

    function __construct ($ini_file = null)
    {
        $this->rgraph_options($ini_file);
    }

    function rgraph_options ($ini_file)
    {
        // common options
        $this->debug = false;
        if ($ini_file && file_exists($ini_file))
        {
            $this->set_optionsviaFile($ini_file);
        }
        else
        {
            $this->debug = false;
            $this->marginTop = 50;
            $this->marginBottom = 50;
            $this->marginLeft = 50;
            $this->marginRight = 50;
            $this->scaleDecimals = 0;
            $this->scalePoint = ".";
            $this->scaleThousand = ",";
            $this->colors = array(
                    "rgb(0,0,255)",
                    "#0f0",
                    "#00f",
                    "#ff0",
                    "#0ff",
                    "#0f0"
            );
        }
    }

    public function get_option_doc ($option)
    {
        $option = strtolower($option);
        foreach (rgraph_valid_options::options() as $key => $value)
        {
            $new_key = strtolower($key);
            if ($new_key == $option)
            {
                return array(
                        $key,
                        $value
                );
                break;
            }
        }
        return - 1;
    }

    public function get_option ($option)
    {
        if (isset($this->$option))
            return $this->$option;
    }

    public function get_valid_options ()
    {
        return rgraph_valid_options::options();
    }

    public function get_valid_option ($option)
    {
        $value = rgraph_valid_options::options($option);
        if ($value != - 1)
            return $value;
        else
            return - 1;
    }

    public function set_option ($option, $parm, $force = false)
    {
        if ($force === true)
        {
            $this->$option = $parm;
            return true;
        }

        $value = rgraph_valid_options::options($option);
        if ($value != - 1)
        {
            $type = gettype($parm);
            if ($type == $value['type'])
            {
                if ($option == "show_labels")
                    $this->labels = $parm;

                if ($option == "xaxisLabels")
                    $this->labels = $parm;

                if ($option == "contextmenu")
                {
                    if (! is_array($parm[0]))
                    {
                        if (isset($parm[1]))
                            $parm[1] = "NO_QUOTE" . $parm[1] . "NO_QUOTE";
                    }
                    else
                    {
                        for ($i = 0; $i < count($parm); $i ++)
                        {
                            if (isset($parm[$i][1]))
                                $parm[$i][1] = "NO_QUOTE" . $parm[$i][1] . "NO_QUOTE";
                        }
                    }
                }
                if ($option == "eventsClick" || $option == "eventsMousemove")
                    $parm = 'NO_QUOTE' . $parm . 'NO_QUOTE';

                if ($option == "fill.click" || $option == "fill.mousemove")
                    $parm = 'NO_QUOTE' . $parm . 'NO_QUOTE';

                if ($option == "tooltipsOverride" || $option == "xscaleFormatter")
                    $parm = 'NO_QUOTE' . $parm . 'NO_QUOTE';

                $this->$option = $parm;

                return true;
            }
            elseif ($type == "integer" && $value['type'] == "double")
            {
                $this->$option = $parm;
                return true;
            }
            elseif ($value['type'] == "double" && is_numeric($parm))
            {
                $this->$option = (float) $parm;
                return true;
            }
            elseif ($value['type'] == "integer" && is_numeric($parm))
            {
                $this->$option = (integer) $parm;
                return true;
            }
            elseif ($value['type'] == "boolean" && strtolower($parm) == "true")
            {
                $this->$option = true;
                return true;
            }
            elseif ($value['type'] == "boolean" && strtolower($parm) == "false")
            {
                $this->$option = false;
            }
            elseif ($value['type'] == "boolean" && $parm == "1")
            {
                $this->$option = true;
                return true;
            }
            elseif ($value['type'] == "boolean" && $parm == "0")
            {
                $this->$option = false;
            }
            elseif ($parm == null)
            {
                $this->$option = null;
            }
            else
                return false;
        }
        if ($this->debug == true && $option != "debug")
            echo "<br>  option " . $option . " not set";
        return - 1;
    }

    public function get_optionType ($option)
    {
        $value = rgraph_valid_options::options($option);
        if ($value != - 1)
            return $value['type'];
        else
            return - 1;
    }

    public function check_option ($option)
    {
        foreach (rgraph_valid_options::options() as $key => $value)
        {
            if (strtolower($key) == strtolower($option))
                return true;
        }
        return - 1;
    }

    public function set_tooltip_formatted ($data, $string = "#val#", $type = "Bar")
    {
        if (! isset($this->tooltipsFormattedThousand))
            $this->tooltipsFormattedThousand = ",";
        if (! isset($this->tooltipsFormattedDecimals))
            $this->tooltipsFormattedDecimals = $this->scaleDecimals;
        if (! isset($this->tooltipsFormattedPoint))
            $this->tooltipsFormattedPoint = ".";
        $num_dec = $this->tooltipsFormattedDecimals;
        $sep_thousand = $this->tooltipsFormattedThousand;
        $sep_dec = $this->tooltipsFormattedPoint;

        if ($type != "Gantt")
            $string = str_replace("#val#", "%{value_formatted}", $string);

        if (isset($data[0]))
        {
            if (! is_array($data[0]) && is_array($data))
            {
                $total = array_sum($data); // $num_dec, $sep_dec,
                                           // $sep_thousand);
                $string = str_replace("#total#", $total, $string);

                for ($i = 0; $i < count($data); $i ++)
                {
                    if ($total != 0)
                        $percent[$i] = number_format(100 * $data[$i] / $total, $num_dec);
                }
            }
            elseif (is_array($data[0]) && $type != "Gantt")
            {
                // calculate totals / group
                for ($i = 0; $i < count($data); $i ++)
                {
                    $total[$i] = 0;
                    for ($j = 0; $j < count($data[$i]); $j ++)
                        $total[$i] += $data[$i][$j]; // $sep_dec,
                                                     // $sep_thousand);
                }
                for ($i = 0; $i < count($total); $i ++)
                    $total[$i] = number_format($total[$i], $num_dec, $sep_dec, $sep_thousand);
            }
        }

        switch ($type)
        {
            case "Gantt":

                for ($i = 0; $i < count($data); $i ++)
                {
                    $start[$i] = number_format($data[$i]['start'], 0);
                    $complete[$i] = number_format($data[$i]['complete'], 0);
                    $duration[$i] = number_format($data[$i]['duration'], 0);
                }

                $this->start = $start;
                $this->duration = $duration;
                $this->complete = $complete;

                $string = str_replace("#val0#", "%{property:start[%{dataset}]}", $string);
                $string = str_replace("#val1#", "%{property:duration[%{dataset}]}", $string);
                $string = str_replace("#val2#", "%{property:complete[%{dataset}]}", $string);
                $string = str_replace("#label#", "%{property:yaxisLabels[%{dataset}]}", $string);
                break;

            case "Bipolar":
                if (! empty($this->key))
                    $string = str_replace("#key#", "%{property:key[%{group}]}", $string);
                $string = str_replace("#label#", "%{property:labels[%{dataset2}]}", $string);
                break;

            case "Pie":
            case "Funnel":
            case "Waterfall":
            case "Line":
            case "Radar":
                if (! empty($this->key))
                    $string = str_replace("#key#", "%{property:key[%{group}]}", $string);
                if (strpos($string, "#percent#") !== false)
                {
                    $this->percent = $percent;
                    $string = str_replace("#percent#", "%{property:percent[%{index}]}", $string);
                }
                if (strpos($string, "#total#") !== false)
                {
                    $this->total = $total;
                    $string = str_replace("#total#", "%{property:total[%{group}]}", $string);
                }

                if (isset($this->labels))
                    $string = str_replace("#label#", "%{property:labels[%{index}]}", $string);
                break;

            default:
                if (! empty($this->key))
                    $string = str_replace("#key#", "%{property:key[%{index}]}", $string);
                if (strpos($string, "#total#") !== false)
                {
                    $this->total = $total;
                    $string = str_replace("#total#", "%{property:total[%{group}]}", $string);
                }
                if (isset($this->labels))
                    $string = str_replace("#label#", "%{property:labels[%{group}]}", $string);
                break;
        }

        $this->tooltips = $string;
        return true;
    }

    public function set_tooltip ($data, $string = "#val#", $group = false)
    {
        if (! is_array($data))
            return - 1;

        if (! isset($this->scaleThousand))
            $this->scaleThousand = ",";
        if (! isset($this->scalePoint))
            $this->scalePoint = ".";
        if (! isset($this->scaleDecimals))
            $this->scaleDecimals = 0;

        $sep_thousand = $this->scaleThousand;
        $sep_dec = $this->scalePoint;
        $num_dec = $this->scaleDecimals;

        if (! is_array($data[0]))
        {
            $total = array_sum($data);
            $tips = array_values($data);

            if (isset($this->labels) && count($this->labels) < count($tips))
            {
                do
                {
                    array_push($this->labels, "");
                }
                while (count($this->labels) < count($tips));
            }

            for ($i = 0; $i < count($tips); $i ++)
            {
                if ($total != 0)
                    $percent = round(100 * $tips[$i] / $total, $num_dec);
                else
                    $percent = "nan";

                $tmp = str_replace("#percent#", $percent, $string);
                if (isset($this->labels[$i]))
                    $tmp = str_replace("#label#", $this->labels[$i], $tmp);
                $tmp = str_replace("#total#", number_format($total, $num_dec, $sep_dec, $sep_thousand), $tmp);
                $tip[$i] = str_replace("#val#", number_format((float) $tips[$i], $num_dec, $sep_dec, $sep_thousand), $tmp);
            }
        }
        else
        {
            // calculate totals / group
            for ($i = 0; $i < count($data); $i ++)
            {
                $total[$i] = 0;
                for ($j = 0; $j < count($data[$i]); $j ++)
                    if (is_numeric($data[$i][$j]))
                        $total[$i] += $data[$i][$j];
            }

            $k = 0;
            if ($group == false)
            {
                for ($i = 0; $i < count($data); $i ++)
                {
                    if (isset($this->labels[$i]))
                        $tmp = str_replace("#label#", $this->labels[$i], $string);
                    else
                        $tmp = $string;

                    for ($j = 0; $j < count($data[$i]); $j ++)
                    {
                        if (isset($this->key[$j]))
                            $tmp2 = str_replace("#key#", $this->key[$j], $tmp);
                        else
                            $tmp2 = $tmp;

                        $tmp2 = str_replace("#total#", number_format($total[$i], $num_dec, $sep_dec, $sep_thousand), $tmp2);
                        $tip[$k] = str_replace("#val#", number_format((float) $data[$i][$j], $num_dec, $sep_dec, $sep_thousand), $tmp2);
                        $k ++;
                    }
                }
            }
            else
            {
                for ($i = 0; $i < count($data); $i ++)
                {
                    for ($j = 0; $j < count($data[$i]); $j ++)
                    {
                        if (isset($this->labels[$j]))
                            $tmp = str_replace("#label#", $this->labels[$j], $string);
                        else
                            $tmp = $string;

                        if (isset($this->key[$i]))
                            $tmp2 = str_replace("#key#", $this->key[$i], $tmp);
                        else
                            $tmp2 = $tmp;

                        $tmp2 = str_replace("#total#", number_format($total[$i], $num_dec, $sep_dec, $sep_thousand), $tmp2);
                        $tip[$k] = str_replace("#val#", number_format((float) $data[$i][$j], $num_dec, $sep_dec, $sep_thousand), $tmp2);
                        $k ++;
                    }
                }
            }
        }

        $this->tooltips = $tip;
        return true;
    }

    public static function xlabelsteps ($labels, $num)
    {
        if (! is_array($labels))
            return false;

        for ($i = 0; $i < count($labels); $i ++)
        {
            if ($i % $num == 0)
                $new_labels[$i] = $labels[$i];
            else
                $new_labels[$i] = "";
        }
        return $new_labels;
    }

    public function get_colors ()
    {
        if (isset($this->colors))
            return $this->colors;
    }

    public function set_gradientColors ()
    {
        for ($i = 0; $i < count($this->colors); $i ++)
        {
            $new_colors[$i] = $this->set_gradientColor($this->colors[$i]);
        }

        $this->colors = $new_colors;
    }

    public function set_gradientColor ($color)
    {
        $gradient = "Gradient(";
        $k = 0.5;
        for ($i = 0; $i < 5; $i ++)
        {
            $rgba = $this->setOpacity($color, $k);
            $gradient .= $rgba . ":";
            $k += 0.2;
        }
        $gradient = rtrim($gradient, ":") . ")";

        return $gradient;
    }

    public function setColorOpacity ($opacity = 0.8)
    {
        if (! is_array($this->colors))
            return false;
        if ($opacity > 1)
            $opacity = 1.0;

        for ($i = 0; $i < count($this->colors); $i ++)
            $this->colors[$i] = $this->setOpacity($this->colors[$i], $opacity);

        return true;
    }

    public static function setOpacity ($color, $opacity = 1.0)
    {
        if (empty($color))
            return false;

        if ($color[0] == '#')
        {
            $color = substr($color, 1);
        }

        if (substr($color, 0, 4) == 'rgb(')
        {
            $tmp = explode(",", $color);
            $tmp[0] = str_replace("rgb", "rgba", $tmp[0]);
            $tmp[2] = str_replace(")", "", $tmp[2]);
            array_push($tmp, $opacity);
            $color = implode(",", $tmp) . ")";
            return ($color);
        }

        if (substr($color, 0, 4) == 'rgba')
        {
            $tmp = explode(",", $color);
            $tmp[3] = $opacity . ")";
            $color = implode(",", $tmp);
            return ($color);
        }

        if (strlen($color) == 6)
        {
            $hex = array(
                    $color[0] . $color[1],
                    $color[2] . $color[3],
                    $color[4] . $color[5]
            );
        }
        elseif (strlen($color) == 3)
        {
            $hex = array(
                    $color[0] . $color[0],
                    $color[1] . $color[1],
                    $color[2] . $color[2]
            );
        }
        else
            return $color;

        $rgb = array_map('hexdec', $hex);

        if (abs($opacity) > 1)
            $opacity = 1.0;
        $rgba_color = 'rgba(' . implode(",", $rgb) . ',' . $opacity . ')';

        return $rgba_color;
    }

    public function set_optionsviaFile ($ini_file)
    {
        if (! file_exists($ini_file))
            return - 1;

        $parms = parse_ini_file($ini_file, false, INI_SCANNER_TYPED);

        if ($parms === false)
            return false;

        $result = true;

        foreach ($parms as $key => $value)
        {
            $type = $this->get_optionType($key);
            if ($type != - 1)
            {
                if ($type == "array")
                {
                    if (strpos($value, "rgb") !== false)
                    {
                        $value = $this->buildcolorsArray($value);
                    }
                    else
                    {
                        if (strpos($value, "|"))
                            $tmp = explode("|", $value);
                        else
                            $value = explode(",", $value);

                        if (isset($tmp))
                        {
                            $value = array();
                            for ($i = 0; $i < count($tmp); $i ++)
                                $value[$i] = explode(",", $tmp[$i]);
                        }
                    }
                }
                $rc = $this->set_option($key, $value);
                if ($rc !== true)
                    $result = 2;
            }
            else
                $result = 2;
        }
        return $result;
    }

    public static function buildcolorsArray ($color_values)
    {
        $tmp = explode(",", $color_values);

        $k = 0;
        $new_values = array();
        for ($i = 0; $i < count($tmp); $i ++)
        {
            if (strpos($tmp[$i], "rgb") !== false)
            {
                $count = 3;

                if (strpos($tmp[$i], "rgba") !== false)
                    $count = 4;

                $rgb_array = array();
                for ($j = 0; $j < $count; $j ++)
                {
                    array_push($rgb_array, $tmp[$i]);
                    $i ++;
                }
                $new_values[$k] = implode(",", $rgb_array);
            }
            else
                $new_values[$k] = $tmp[$i];
            $k ++;
        }

        return $new_values;
    }

    public static function truncateLabels ($labels, $size, $chars = null)
    {
        if ($size == '0')
            return $labels;

        if (! is_array($labels))
            return false;

        for ($i = 0; $i < count($labels); $i ++)
        {
            if (strlen($labels[$i]) > $size)
                $labels[$i] = mb_substr($labels[$i], 0, $size) . $chars;
        }

        return $labels;
    }

    public static function getRandomColor ()
    {
        $hex_rgb = dechex(rand(0, 255)) . dechex(rand(0, 255)) . dechex(rand(0, 255));
        if (strlen($hex_rgb) == 4)
        {
            $hex_rgb = $hex_rgb . rand(10, 99);
        }
        elseif (strlen($hex_rgb) == 5)
        {
            $hex_rgb = $hex_rgb . rand(0, 9);
        }
        $color = "#" . $hex_rgb;

        return $color;
    }

    public static function getRandomColors ($count = 1)
    {
        $colors = array();

        for ($i = 0; $i < $count; $i ++)
            $colors[$i] = getRandomColor();

        return $colors;
    }
}

?>